
// TestSocket.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "VaxPipeClient.h"

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

CVaxPipeClient::CVaxPipeClient()
{
	m_hHandlePipe = NULL;
    m_pWriteDataList = NULL;
    
    m_hWaitEventRead = NULL;
    m_hWaitEventWrite = NULL;
}

CVaxPipeClient::~CVaxPipeClient()
{
    ClosePipe();
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

BOOLEAN CVaxPipeClient::CreatePipeReadOnly(LPCTSTR sPipeName, HANDLE& hWaitEventRead)
{
    HANDLE hWaitEventWrite = NULL;

    return CreatePipe(sPipeName, FALSE, hWaitEventRead, hWaitEventWrite);
}

BOOLEAN CVaxPipeClient::CreatePipeReadWrite(LPCTSTR sPipeName, HANDLE& hWaitEventRead, HANDLE& hWaitEventWrite)
{
    return CreatePipe(sPipeName, TRUE, hWaitEventRead, hWaitEventWrite);
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

BOOLEAN CVaxPipeClient::CreatePipe(LPCTSTR sPipeName, BOOL bEnableWrite, HANDLE& hWaitEventRead, HANDLE& hWaitEventWrite)
{
    DWORD dwOpenMode = GENERIC_READ | FILE_WRITE_ATTRIBUTES;

    if (bEnableWrite)
    {
        dwOpenMode = GENERIC_READ | GENERIC_WRITE | FILE_WRITE_ATTRIBUTES;
    }

    m_hHandlePipe = ::CreateFile(sPipeName, dwOpenMode, 0, NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);

    if (m_hHandlePipe == INVALID_HANDLE_VALUE)
    {
        if (::GetLastError() != ERROR_PIPE_BUSY)
            return FALSE;

        if (!::WaitNamedPipe(sPipeName, 3000))
            return FALSE;
    }

    DWORD dwMode = PIPE_READMODE_MESSAGE;

    ::SetNamedPipeHandleState(m_hHandlePipe, &dwMode, NULL, NULL);

    m_hWaitEventRead = CreateEvent(NULL, FALSE, FALSE, NULL);

    if (m_hWaitEventRead == NULL)
    {
        ClosePipe();
        return FALSE;
    }
    
    if (bEnableWrite)
    {
        m_hWaitEventWrite = ::CreateEvent(NULL, FALSE, FALSE, NULL);

        if (m_hWaitEventWrite == NULL)
        {
            ClosePipe();
            return FALSE;
        }

        m_pWriteDataList = new CVaxPipeDataList();
    }

    hWaitEventRead = m_hWaitEventRead;
    hWaitEventWrite = m_hWaitEventWrite;
    
    return TRUE;
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

void CVaxPipeClient::ClosePipe()
{
    if (m_pWriteDataList)
    {
        while (!m_pWriteDataList->IsEmpty())
            delete m_pWriteDataList->RemoveHead();

        delete m_pWriteDataList;
        m_pWriteDataList = NULL;
    }

    if (m_hHandlePipe != NULL)
    {
        ::CloseHandle(m_hHandlePipe);
        m_hHandlePipe = NULL;
    }

    if (m_hWaitEventRead != NULL)
    {
        ::CloseHandle(m_hWaitEventRead);
        m_hWaitEventRead = NULL;
    }

    if (m_hWaitEventWrite != NULL)
    {
        ::CloseHandle(m_hWaitEventWrite);
        m_hWaitEventWrite = NULL;
    }
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

BOOLEAN CVaxPipeClient::BeginReadPipe()
{
    ::memset(&m_hOverlapRead, 0, sizeof(OVERLAPPED));
    m_hOverlapRead.hEvent = m_hWaitEventRead;

    BOOLEAN bResult = ReadFile(m_hHandlePipe, m_aDataReadPipe, VAX_PIPE_READ_BUFFER_LENGTH, NULL, &m_hOverlapRead);
    
    if (bResult == FALSE)
    {
        int nError = ::GetLastError();

        if ((nError == ERROR_IO_PENDING) || (nError == ERROR_MORE_DATA))
            return TRUE;
                
        return FALSE;
    }

    return TRUE;
}

BOOLEAN CVaxPipeClient::EndReadPipe(char* pDataRead, int& nSizeRead)
{
    DWORD dwSizeRead = 0;

    if (!::GetOverlappedResult(m_hHandlePipe, &m_hOverlapRead, &dwSizeRead, FALSE))
        return FALSE;

    nSizeRead = dwSizeRead;

    if(dwSizeRead != 0)
        ::memcpy(pDataRead, m_aDataReadPipe, dwSizeRead);

    return TRUE;
}

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////

#define PIPE_MAX_NO_DATA_MESSAGES    240

void CVaxPipeClient::BeginWritePipe(char* pData, int nSize)
{
    if (m_hHandlePipe == NULL || m_hWaitEventWrite == NULL)
        return;

    if (m_pWriteDataList->GetCount() > PIPE_MAX_NO_DATA_MESSAGES)
        return;

    CVaxPipeData* pDataWrite = new CVaxPipeData(pData, nSize);

    if (!m_pWriteDataList->IsEmpty())
    {
        m_pWriteDataList->AddTail(pDataWrite);
        return;
    }

    if (!WritePipe(pDataWrite->m_pData, pDataWrite->m_nSize))
    {
        delete pDataWrite;
        return;
    }

    m_pWriteDataList->AddTail(pDataWrite);
}

BOOLEAN CVaxPipeClient::WritePipe(char* pData, int nSize)
{
    if (m_hHandlePipe == NULL || m_hWaitEventWrite == NULL)
        return FALSE;

    ::memset(&m_hOverlapWrite, 0, sizeof(OVERLAPPED));
    m_hOverlapWrite.hEvent = m_hWaitEventWrite;

    BOOLEAN bResult = ::WriteFile(m_hHandlePipe, pData, nSize, NULL, &m_hOverlapWrite);

    if (bResult == FALSE)
    {
        int nError = ::GetLastError();

        if (nError == ERROR_IO_PENDING)
        {
            return TRUE;
        }

        if ((nError == ERROR_BROKEN_PIPE) || (nError == ERROR_NO_DATA))
        {
            
        }

        return FALSE;
    }

    return TRUE;
}

void CVaxPipeClient::EndWritePipe()
{
    if (m_hHandlePipe == NULL || m_hWaitEventWrite == NULL)
        return;

    if (m_pWriteDataList->IsEmpty())
        return;

    delete m_pWriteDataList->RemoveHead();

    if (m_pWriteDataList->IsEmpty())
        return;

    CVaxPipeData* pDataWrite = m_pWriteDataList->GetHead();

    if (!WritePipe(pDataWrite->m_pData, pDataWrite->m_nSize))
    {
        return;
    }
}

///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////