#include "StdAfx.h"

#include "VaxCallSession.h"
#include "VaxServerSIP.h"

#include "VaxAccessAudioPCM.h"
#include "VaxCall.h"
#include "VaxTransfer.h"

#include "VaxDTMF.h"
#include "VaxDetectAMD.h"

#include "VaxCallParking.h"
#include "VaxRoom.h"
#include "VaxCallSessionRoute.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

CVaxCallSessionRoute* CVaxCallSession::m_pRouteCallSIP = NULL;

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

CVaxCallSession::CVaxCallSession()
{
	if(CVaxCallSession::m_pRouteCallSIP == NULL)
		CVaxCallSession::m_pRouteCallSIP = new CVaxCallSessionRoute();
			
	m_pAccessPCM = new CVaxAccessAudioPCM();

	m_pVaxCall = new CVaxCall();
	m_pVaxTransfer = new CVaxTransfer();
		
	m_pVaxDTMF = new CVaxDTMF();
	m_pVaxAMD = new CVaxDetectAMD();
	m_pVaxRoom = new CVaxRoom();
}

CVaxCallSession::~CVaxCallSession()
{
	delete m_pAccessPCM;

	delete m_pVaxCall;
	delete m_pVaxTransfer;
		
	delete m_pVaxDTMF;
	delete m_pVaxAMD;
		
	delete m_pVaxRoom;
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnCallSessionClosed(int nSessionId, int nChannelId, int nReasonCode)
{
	m_pAccessPCM->OnCallSessionClosed(nSessionId);
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::AddUserRoute(LPCTSTR sUserName, LPCTSTR sUserPrefix, LPCTSTR sDialPrefix, LPCTSTR sToPeerName)
{
	CVaxCallSession::m_pRouteCallSIP->AddUserRoute(sUserName, sUserPrefix, sDialPrefix, sToPeerName);
}

void CVaxCallSession::RemoveUserRoute(LPCTSTR sUserName)
{
	CVaxCallSession::m_pRouteCallSIP->RemoveUserRoute(sUserName);
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::AddLineRoute(LPCTSTR sLineName, LPCTSTR sToPeerName, BOOL bRejected, LPCTSTR sRoomName)
{
	CVaxCallSession::m_pRouteCallSIP->AddLineRoute(sLineName, sToPeerName, bRejected, sRoomName);
}

void CVaxCallSession::RemoveLineRoute(LPCTSTR sLineName)
{
	CVaxCallSession::m_pRouteCallSIP->RemoveLineRoute(sLineName);
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::ResetAll(BOOL bAlloc, BOOL bDeAlloc)
{
	if(bDeAlloc)
	{
		if(CVaxCallSession::m_pRouteCallSIP != NULL)
		{
			delete CVaxCallSession::m_pRouteCallSIP;
			CVaxCallSession::m_pRouteCallSIP = NULL;
		}
	}

	if(bAlloc) 
		CVaxCallSession::m_pRouteCallSIP = new CVaxCallSessionRoute();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////
       
void CVaxCallSession::GetFromPeerRouteInfo(LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sDialNo, CString& sDialToNo, CString& sToPeerName, BOOL& bRejected, CString& sRoomName)
{
	if (nFromPeerType == VAX_PEER_TYPE_USER)
	{
		CVaxCallSession::m_pRouteCallSIP->GetUserRouteInfo(sFromPeerName, sDialNo, sDialToNo, sToPeerName);
		return;
	}
		
	CVaxCallSession::m_pRouteCallSIP->GetLineRouteInfo(sFromPeerName, sDialNo, sDialToNo, sToPeerName, bRejected, sRoomName);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnIncomingCall(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sDialNo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sUserAgentName, LPCTSTR sFromIP, LONG nFromPort)
{
	CString sToPeerName, sDialToNo;
    CString sRoomName;
	BOOL bRejected = FALSE;

	GetFromPeerRouteInfo(nFromPeerType, sFromPeerName, sDialNo, sDialToNo, sToPeerName, bRejected, sRoomName);

	if(bRejected)
	{
		CVaxServerSIP::m_pVaxSIP->RejectCallSession(nSessionId, 404, "Not Found");
		return;
	}

	m_pAccessPCM->OnIncomingCall(nSessionId);

	OnProcessCall(nSessionId, sCallerName, sCallerId, sDialToNo, nFromPeerType, sFromPeerName, sUserAgentName, sFromIP, nFromPort, sToPeerName, sRoomName);
}

///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnProcessCall(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sDialNo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sUserAgentName, LPCTSTR sFromIP, LONG nFromPort, LPCTSTR sToPeerName, LPCTSTR sRoomName)
{
	BOOL bProcessed = FALSE;

	m_pVaxRoom->OnIncomingCall(nSessionId, sRoomName, bProcessed);
    if(bProcessed) return;
	    	
	m_pVaxCall->OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, nFromPeerType, sFromPeerName, sToPeerName, bProcessed); // Process as normal call.
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnCallSessionTransferBlind(LONG nTransfererSessionId, LONG nTransfererChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo)
{
	m_pVaxTransfer->OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);
}

void CVaxCallSession::OnCallSessionTransferConsult(LONG nTransfererSessionId, LONG nTransfererChannelId, LONG nTransferToSessionId, LONG nTransferToChannelId, LPCTSTR sTransferer, LPCTSTR sTransferee, LPCTSTR sTransferTo)
{
	m_pVaxTransfer->OnCallSessionTransferConsult(nTransfererSessionId, nTransfererChannelId, nTransferToSessionId,  nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnDetectedDigitDTMF(LONG nSessionId, LONG nChannelId, LPCTSTR sDigitDTMF)
{
	m_pVaxDTMF->OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
}

////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnCallSessionDetectedAMD(LONG nSessionId, LONG nChannelId, BOOL bIsHuman)
{
	m_pVaxAMD->OnCallSessionDetectedAMD(nSessionId, nChannelId, bIsHuman);
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

void CVaxCallSession::OnCallSessionAccessAudioPCM(LONG nSessionId, LONG nChannelId, LONG nAccessType, char* pDataPCM, LONG nSizePCM)
{
	m_pAccessPCM->OnCallSessionAccessAudioPCM(nSessionId, nChannelId, nAccessType, pDataPCM, nSizePCM);
}
